@@ -15,6 +15,7 @@ use internal::dirs::{
     impl_concretizations_directory_from_target, output_directory_from_target,
     queue_directory_from_target, target_directory,
 };
+use lazy_static::lazy_static;
 use log::debug;
 use semver::VersionReq;
 use serde::{Deserialize, Serialize};
@@ -26,6 +27,7 @@ use std::{
     iter,
     path::{Path, PathBuf},
     process::{exit, Command},
+    sync::Mutex,
     time::Duration,
 };
 use subprocess::{CommunicateError, Exec, ExitStatus, NullFile, Redirection};
@@ -63,111 +65,120 @@ enum SubCommand {
 #[derive(Clone, Debug, Deserialize, Parser, Serialize)]
 #[clap(version = crate_version!())]
 struct TestFuzz {
-    #[clap(long, about = "Display backtraces")]
+    #[clap(long, help = "Display backtraces")]
     backtrace: bool,
     #[clap(
         long,
-        about = "Move one target's crashes, hangs, and work queue to its corpus; to consolidate \
+        help = "Move one target's crashes, hangs, and work queue to its corpus; to consolidate \
         all targets, use --consolidate-all"
     )]
     consolidate: bool,
-    #[clap(long, hidden = true)]
+    #[clap(long, hide = true)]
     consolidate_all: bool,
-    #[clap(long, about = "Display concretizations")]
+    #[clap(long, help = "Display concretizations")]
     display_concretizations: bool,
     #[clap(
         long,
-        about = "Display corpus using uninstrumented fuzz target; to display with instrumentation, \
+        help = "Display corpus using uninstrumented fuzz target; to display with instrumentation, \
         use --display-corpus-instrumented"
     )]
     display_corpus: bool,
-    #[clap(long, hidden = true)]
+    #[clap(long, hide = true)]
     display_corpus_instrumented: bool,
-    #[clap(long, about = "Display crashes")]
+    #[clap(long, help = "Display crashes")]
     display_crashes: bool,
-    #[clap(long, about = "Display hangs")]
+    #[clap(long, help = "Display hangs")]
     display_hangs: bool,
-    #[clap(long, about = "Display `impl` concretizations")]
+    #[clap(long, help = "Display `impl` concretizations")]
     display_impl_concretizations: bool,
-    #[clap(long, about = "Display work queue")]
+    #[clap(long, help = "Display work queue")]
     display_queue: bool,
-    #[clap(long, about = "Target name is an exact name rather than a substring")]
+    #[clap(long, help = "Target name is an exact name rather than a substring")]
     exact: bool,
     #[clap(
         long,
-        about = "Exit with 0 if the time limit was reached, 1 for other programmatic aborts, and 2 \
-        if an error occurred; implies --no-ui, does not imply --run-until-crash or -- -V <seconds>"
+        help = "Exit with 0 if the time limit was reached, 1 for other programmatic aborts, and 2 \
+        if an error occurred; implies --no-ui, does not imply --run-until-crash or -- -V <SECONDS>"
     )]
     exit_code: bool,
     #[clap(
         long,
         multiple_occurrences = true,
-        about = "Space or comma separated list of features to activate"
+        help = "Space or comma separated list of features to activate"
     )]
     features: Vec<String>,
-    #[clap(long, about = "List fuzz targets")]
+    #[clap(long, help = "List fuzz targets")]
     list: bool,
-    #[clap(long, value_name = "PATH", about = "Path to Cargo.toml")]
+    #[clap(long, value_name = "PATH", help = "Path to Cargo.toml")]
     manifest_path: Option<String>,
-    #[clap(long, about = "Do not activate the `default` feature")]
+    #[clap(long, help = "Do not activate the `default` feature")]
     no_default_features: bool,
     #[clap(
         long,
-        about = "Compile without instrumentation (for testing build process)"
+        help = "Compile without instrumentation (for testing build process)"
     )]
     no_instrumentation: bool,
-    #[clap(long, about = "Compile, but don't fuzz")]
+    #[clap(long, help = "Compile, but don't fuzz")]
     no_run: bool,
-    #[clap(long, about = "Disable user interface")]
+    #[clap(long, help = "Disable user interface")]
     no_ui: bool,
-    #[clap(short, long, about = "Package containing fuzz target")]
+    #[clap(short, long, help = "Package containing fuzz target")]
     package: Option<String>,
-    #[clap(long, about = "Enable persistent mode fuzzing")]
+    #[clap(long, help = "Enable persistent mode fuzzing")]
     persistent: bool,
-    #[clap(long, about = "Pretty-print debug output when displaying/replaying")]
+    #[clap(long, help = "Pretty-print debug output when displaying/replaying")]
     pretty_print: bool,
     #[clap(
         long,
-        about = "Replay corpus using uninstrumented fuzz target; to replay with instrumentation, \
+        help = "Replay corpus using uninstrumented fuzz target; to replay with instrumentation, \
         use --replay-corpus-instrumented"
     )]
     replay_corpus: bool,
-    #[clap(long, hidden = true)]
+    #[clap(long, hide = true)]
     replay_corpus_instrumented: bool,
-    #[clap(long, about = "Replay crashes")]
+    #[clap(long, help = "Replay crashes")]
     replay_crashes: bool,
-    #[clap(long, about = "Replay hangs")]
+    #[clap(long, help = "Replay hangs")]
     replay_hangs: bool,
-    #[clap(long, about = "Replay work queue")]
+    #[clap(long, help = "Replay work queue")]
     replay_queue: bool,
     #[clap(
         long,
-        about = "Clear fuzzing data for one target, but leave corpus intact; to reset all \
+        help = "Clear fuzzing data for one target, but leave corpus intact; to reset all \
         targets, use --reset-all"
     )]
     reset: bool,
-    #[clap(long, hidden = true)]
+    #[clap(long, hide = true)]
     reset_all: bool,
-    #[clap(long, about = "Resume target's last fuzzing session")]
+    #[clap(long, help = "Resume target's last fuzzing session")]
     resume: bool,
-    #[clap(long, about = "Stop fuzzing once a crash is found")]
+    #[clap(long, help = "Stop fuzzing once a crash is found")]
     run_until_crash: bool,
-    #[clap(long, about = "String that fuzz target's name must contain")]
+    #[clap(
+        long,
+        value_name = "TARGETNAME",
+        help = "DEPRECATED: Use just <TARGETNAME>"
+    )]
     target: Option<String>,
     #[clap(
         long,
         value_name = "NAME",
-        about = "Integration test containing fuzz target"
+        help = "Integration test containing fuzz target"
     )]
     test: Option<String>,
     #[clap(
         long,
-        about = "Number of milliseconds to consider a hang when fuzzing or replaying (equivalent \
-        to `-- -t <timeout>` when fuzzing)"
+        help = "Number of milliseconds to consider a hang when fuzzing or replaying (equivalent \
+        to -- -t <TIMEOUT> when fuzzing)"
     )]
     timeout: Option<u64>,
-    #[clap(last = true, name = "args", about = "Arguments for the fuzzer")]
-    zargs: Vec<String>,
+    #[clap(
+        value_name = "TARGETNAME",
+        help = "String that fuzz target's name must contain"
+    )]
+    ztarget: Option<String>,
+    #[clap(last = true, name = "args", help = "Arguments for the fuzzer")]
+    zzargs: Vec<String>,
 }
 
 #[derive(Clone, Deserialize, Serialize)]
@@ -213,9 +224,15 @@ pub fn cargo_test_fuzz<T: AsRef<OsStr>>(args: &[T]) -> Result<()> {
         {
             opts.no_instrumentation = true;
         }
+        if let Some(target_name) = opts.target.take() {
+            eprintln!("`--target <TARGETNAME>` is deprecated. Use just `<TARGETNAME>`.");
+            opts.ztarget = opts.ztarget.or(Some(target_name));
+        }
         opts
     };
 
+    cache_cargo_afl_version()?;
+
     let display = opts.display_corpus
         || opts.display_corpus_instrumented
         || opts.display_crashes
@@ -234,7 +251,7 @@ pub fn cargo_test_fuzz<T: AsRef<OsStr>>(args: &[T]) -> Result<()> {
 
     let mut executable_targets = executable_targets(&executables)?;
 
-    if let Some(pat) = &opts.target {
+    if let Some(pat) = &opts.ztarget {
         executable_targets = filter_executable_targets(&opts, pat, &executable_targets);
     }
 
@@ -406,7 +423,7 @@ fn build(opts: &TestFuzz, quiet: bool) -> Result<Vec<Executable>> {
         bail!("Command failed: {:?}", exec);
     }
 
-    Ok(artifacts
+    let executables = artifacts
         .into_iter()
         .map(|artifact| {
             if let Artifact {
@@ -429,10 +446,8 @@ fn build(opts: &TestFuzz, quiet: bool) -> Result<Vec<Executable>> {
                 Ok(None)
             }
         })
-        .collect::<Result<Vec<_>>>()?
-        .into_iter()
-        .flatten()
-        .collect())
+        .collect::<Result<Vec<_>>>()?;
+    Ok(executables.into_iter().flatten().collect())
 }
 
 fn metadata(opts: &TestFuzz) -> Result<Metadata> {
@@ -455,12 +470,12 @@ fn test_fuzz_and_afl_versions(
     let afl = test_fuzz
         .as_ref()
         .map(|package_id| package_dependency(metadata, package_id, "afl"))
-        .transpose()?
-        .flatten();
+        .transpose()?;
     let test_fuzz_version = test_fuzz
         .map(|package_id| package_version(metadata, &package_id))
         .transpose()?;
     let afl_version = afl
+        .flatten()
         .map(|package_id| package_version(metadata, &package_id))
         .transpose()?;
     Ok((test_fuzz_version, afl_version))
@@ -480,14 +495,17 @@ fn package_dependency(
         .iter()
         .find(|node| node.id == *package_id)
         .ok_or_else(|| anyhow!("Could not find package `{}`", package_id))?;
-    Ok(node
+    let package_ids_and_names = node
         .dependencies
         .iter()
-        .map(|package_id| package_name(metadata, package_id).map(|s| (package_id, s == name)))
-        .collect::<Result<Vec<_>>>()?
+        .map(|package_id| {
+            package_name(metadata, package_id).map(|package_name| (package_id, package_name))
+        })
+        .collect::<Result<Vec<_>>>()?;
+    Ok(package_ids_and_names
         .into_iter()
-        .find_map(|(package_id, found)| {
-            if found {
+        .find_map(|(package_id, package_name)| {
+            if package_name == name {
                 Some(package_id.clone())
             } else {
                 None
@@ -622,7 +640,7 @@ fn executable_target(
 }
 
 fn match_message(opts: &TestFuzz) -> String {
-    opts.target.as_ref().map_or("".to_owned(), |pat| {
+    opts.ztarget.as_ref().map_or("".to_owned(), |pat| {
         format!(
             " {} `{}`",
             if opts.exact { "equal to" } else { "containing" },
@@ -631,11 +649,11 @@ fn match_message(opts: &TestFuzz) -> String {
     })
 }
 
+#[allow(clippy::expect_used)]
 fn check_test_fuzz_and_afl_versions(
     executable_targets: &[(Executable, Vec<String>)],
 ) -> Result<()> {
     let cargo_test_fuzz_version = Version::parse(crate_version!())?;
-    let cargo_afl_version = cargo_afl_version()?;
     for (executable, _) in executable_targets {
         check_dependency_version(
             &executable.name,
@@ -650,13 +668,30 @@ fn check_test_fuzz_and_afl_versions(
             "afl",
             executable.afl_version.as_ref(),
             "cargo-afl",
-            &cargo_afl_version,
+            CARGO_AFL_VERSION
+                .lock()
+                .expect("Could not lock `CARGO_AFL_VERSION`")
+                .as_ref()
+                .expect("Could not determine `cargo-afl` version"),
             "afl",
         )?;
     }
     Ok(())
 }
 
+fn cache_cargo_afl_version() -> Result<()> {
+    let cargo_afl_version = cargo_afl_version()?;
+    let mut lock = CARGO_AFL_VERSION
+        .lock()
+        .map_err(|error| anyhow!(error.to_string()))?;
+    *lock = Some(cargo_afl_version);
+    Ok(())
+}
+
+lazy_static! {
+    static ref CARGO_AFL_VERSION: Mutex<Option<Version>> = Mutex::new(None);
+}
+
 fn cargo_afl_version() -> Result<Version> {
     let mut command = Command::new("cargo");
     command.args(&["afl", "--version"]);
@@ -1005,7 +1040,7 @@ fn fuzz(opts: &TestFuzz, executable: &Executable, target: &str) -> Result<()> {
             format!("{}", timeout * NANOS_PER_MILLI),
         ]);
     }
-    args.extend(opts.zargs.clone());
+    args.extend(opts.zzargs.clone());
     args.extend(
         vec![
             "--",
@@ -1093,7 +1128,6 @@ mod tests {
             &("test-fuzz/".to_owned() + test_fuzz::serde_format().as_feature()),
             "--no-run",
             "--no-instrumentation",
-            "--target",
             "target",
         ])
         .unwrap();
