@@ -2,7 +2,7 @@ use internal::{dirs::corpus_directory_from_target, serde_format};
 use lazy_static::lazy_static;
 use predicates::prelude::*;
 use std::{fs::remove_dir_all, sync::Mutex};
-use test_env_log::test;
+use test_log::test;
 use testing::{examples, retry};
 
 const TIMEOUT: &str = "60";
