@@ -2,7 +2,7 @@ use internal::dirs::corpus_directory_from_target;
 use predicates::prelude::*;
 use rlimit::Resource;
 use std::fs::remove_dir_all;
-use test_env_log::test;
+use test_log::test;
 use testing::{examples, retry};
 
 // smoelius: MEMORY_LIMIT must be large enough for the build process to complete.
@@ -27,7 +27,7 @@ fn replay_crashes() {
             &format!("{}", MEMORY_LIMIT / 1024),
         ],
         &What::Crashes,
-        r"memory allocation of \d{10,} bytes failed\n",
+        r"(?m)\bmemory allocation of \d{10,} bytes failed$",
     );
 }
 
@@ -39,7 +39,7 @@ fn replay_hangs() {
         "parse",
         &["--persistent", "--", "-V", TIMEOUT],
         &What::Hangs,
-        r"Timeout\n",
+        r"(?m)\bTimeout$",
     );
 }
 
